import requests
import json
from os.path import expanduser
from wfw.wfexceptions import LoginFailedException
from wfw.tree import Tree


WFURL = 'https://workflowy.com/'
TREE_DATA = expanduser('~/.wfwtree')


def log_in(email, password):
    info = {'username' : email, 'password': password, 'next' : ''}
    request = requests.post(WFURL + 'accounts/login/', data=info)

    if not len(request.history) == 1:
        raise LoginFailedException("Login was not successful")

    cookies = requests.utils.dict_from_cookiejar(request.history[0].cookies)
    return cookies['sessionid']


def log_out(session_id):
    cookie = {'sessionid' : session_id}
    requests.get(WFURL + 'offline_logout', cookies=cookie)


def get_list_from_server(session_id):
    cookie = {'sessionid' : session_id}
    request = requests.post(WFURL + 'get_initialization_data?client_version=14',
                            cookies=cookie)

    tree = request.json()
    with open(TREE_DATA, 'w') as tree_data:
        json.dump(tree, tree_data)


def build_tree_from_file():
    tree = Tree()
    with open(TREE_DATA, 'r') as tree_data:
        tree.build(tree_data)
        return tree


def fetch_list(email, password):
    try:
        session_id = log_in(email, password)
    except LoginFailedException:
        raise
    else:
        get_list_from_server(session_id)
        log_out(session_id)


def print_list(depth, root=None):
    tree = build_tree_from_file()
    if root is None:
        tree.print_subtree(tree.root, depth)
    else:
        tree.print_tree(root, depth)


def export_list(file_name):
    tree = build_tree_from_file()
    tree.export_tree(file_name)


def search_tags(tag):
    tree = build_tree_from_file()
    tree.print_nodes_with_tag(tag)



import json

class Tree(object):

    def __init__(self):
        self.root = Node(0, 'My list', None)


    def __add_node(self, node, parent):
        new_node = Node(node['id'], node['nm'].encode('utf-8'), parent)
        parent.add_child(new_node)

        if 'ch' in node.keys():
            for child in node['ch']:
                self.__add_node(child, new_node)


    def __find_node(self, node, name_to_find):
        if node.name == name_to_find.encode('utf-8'):
            return node

        for child in node.children:
            result = self.__find_node(child, name_to_find)
            if result:
                return result


    def __find_tag(self, node, tag, result=[]):
        for child in node.children:
            if tag.encode('utf-8') in child.name:
                result.append(child)
            self.__find_tag(child, tag, result)

        return result


    def __write_to_file(self, destination, start, depth=0):
        destination.write(start.exportable_format(depth))

        for child in start.children:
            self.__write_to_file(destination, child, depth+1)


    def build(self, input_file):
        raw_tree = json.load(input_file)
        children_of_root = raw_tree['projectTreeData']['mainProjectTreeInfo']['rootProjectChildren']

        for child in children_of_root:
            self.__add_node(child, self.root)


    def print_subtree(self, start, depth, current_depth=0):
        print(start.printable_format(current_depth))

        if depth > current_depth:
            current_depth += 1
            for child in start.children:
                self.print_subtree(child, depth, current_depth)


    def print_tree(self, name, depth):
        root = self.__find_node(self.root, name)

        if not root is None:
            self.print_subtree(root, depth)


    def export_tree(self, file_name):
        with open(file_name, 'w') as destination:
            self.__write_to_file(destination, self.root)


    def print_nodes_with_tag(self, tag):
        result = self.__find_tag(self.root, tag)

        for node in result:
            print(node.printable_format())


class Node(object):

    def __init__(self, node_id, name, parent):
        self.node_id = node_id
        self.name = name
        self.parent = parent
        self.children = []


    def add_child(self, child):
        self.children.append(child)


    def exportable_format(self, depth):
        return "{fill}{name}\n".format(fill='\t' * depth, name=self.name)


    def printable_format(self, depth=0):
        return "{fill}* {name}".format(fill='    ' * depth, name=self.name)



import click
from wfw.wfexceptions import LoginFailedException
from wfw.lib import fetch_list, export_list, print_list, search_tags

@click.group()
def cli():
    pass

@cli.command()
@click.option('--email', prompt=True)
@click.option('--password', prompt=True, hide_input=True)
def fetch(email, password):
    """Fetch list from WorkFlowy server"""

    try:
        fetch_list(email, password)
    except LoginFailedException:
        click.echo("Unable to fetch your list: wrong e-mail or password.")


@cli.command()
@click.argument('file-name', default='tree.exported')
def export(file_name):
    """Export your list"""

    try:
        export_list(file_name)
    except IOError:
        click.echo("You have no local tree")
    except Exception as ex:
        click.echo("Error while exporting: {msg}".format(msg=ex.message))


@cli.command()
@click.argument('level', default=1)
@click.option('--root', default=None)
def show(level, root):
    """Print your list"""

    try:
        print_list(level, root)
    except IOError:
        click.echo("You have no local tree")
    except Exception as ex:
        click.echo("Error while printing list: {msg}".format(msg=ex.message))


@cli.command()
@click.argument('tag-to-find')
def tag(tag_to_find):
    """Find items containing the given tag"""

    try:
        search_tags(tag_to_find)
    except IOError:
        click.echo("You have no local tree")
    except Exception as ex:
        click.echo("Error while searching tag: {msg}".format(msg=ex.message))



from distutils.core import setup
import wfw

setup(
    name='wfw',
    packages=['wfw'],
    version='0.1.3',
    description='command-line interface for WorkFlowy',
    author='Noemi Vanyi',
    author_email='sitbackandwait@gmail.com',
    url='https://github.com/kvch/wfw',
    download_url='https://github.com/kvch/wfw/tarball/0.1.3',
    install_requires=[
        'click',
        'requests',
    ],
    entry_points={
        'console_scripts' : [ 
            'wfw = wfw.__main__:cli'
        ],
    })
